/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
/* the portable parts of mog_queue, epoll/kqueue-agnostic */
#include "cmogstored.h"
#include "activeq.h"

/*
 * activeq is _only_ used from mog_queue_loop.  mog_accept_loop
 * never touches this, it can only safely push to the regular
 * (epoll/kqueue) queue.
 */
void mog_activeq_push(struct mog_queue *q, struct mog_fd *mfd)
{
	if (mfd->queue_state == MOG_QUEUE_STATE_NEW) {
		/*
		 * accept loop can't push into active queue without
		 * potential deadlock (or extra syscalls), so just push
		 * into the idle queue to transition mfd->queue_state
		 */
		mog_idleq_push(q, mfd, MOG_QEV_RW);
	} else {
		mog_activeq_insert(q, mfd);
	}
}

struct mog_fd *mog_activeq_trytake(struct mog_queue *q)
{
	struct mog_fd *mfd;

	CHECK(int, 0, pthread_mutex_lock(&q->activeq_lock));
	mfd = SIMPLEQ_FIRST(&q->activeq_head);
	if (mfd)
		SIMPLEQ_REMOVE_HEAD(&q->activeq_head, active_fd);
	CHECK(int, 0, pthread_mutex_unlock(&q->activeq_lock));

	if (mfd)
		mog_fd_check_out(mfd);

	return mfd;
}
